import java.util.ArrayList;
/**
 * Diese Klasse repräsentiert eine Melodie, die aus mehreren Noten besteht. Die
 * Noten werden in geeigneter Form gespeichert. Verschiedene Methoden
 * ermöglichen das Hinzufügen von Noten, das Kopieren von Notenfolgen und das
 * Transponieren der Melodie. Diese Klasse liest weder direkt von System.in ein,
 * noch gibt sie direkt auf System.out aus.
 */
public class Melody {

	private int bpm;
	ArrayList<Note> melody;

	/**
	 * erzeugt eine neue Instanz von Melody und speichert das übergebene Tempo
	 * (bpm, Schläge pro Minute).
	 * 
	 * @param bpm
	 *            Das Tempo in Schlägen pro Minute
	 */
	public Melody(int bpm) {
		this.melody = new ArrayList<Note>();	// this. mozda nije potrebno 
		this.bpm = bpm;
	}

	/**
	 * fügt am Ende der Melodie die übergebene Note hinzu.
	 * 
	 * @param note
	 *            Die hinzuzufügende Note
	 */
	public void addNote(Note note) {
		this.melody.add(note);		// this. mozda nije potrebno
	}

	/**
	 * kopiert eine Folge von Noten und gibt diese als neue Melodie (Melody)
	 * zurück. Die neue Melodie hat dabei das gleiche Tempo, die kopierten Noten
	 * sollen echte Kopien der ursprünglichen Noten sein. Die neue Melodie
	 * beinhaltet die Noten zwischen inklusive beginIndex und exklusive
	 * endIndex, hat also die Länge endIndex - beginIndex. Besteht die
	 * gespeicherte Melodie beispielsweise aus den Tönen do - re - mi - fa -
	 * sol, so lautet die neue Melodie bei Aufruf der Methode mit den Parametern
	 * beginIndex=1 und endIndex=4 re - mi - fa und hat die Länge 3.
	 * 
	 * @param beginIndex
	 *            Der erste Index der ersten enthaltenen Note
	 * @param endIndex
	 *            Der Index der ersten nicht mehr enthaltenen Note
	 * @return Eine neue Melody aus dem spezifizierten Notenbereich
	 */
	public Melody copy(int beginIndex, int endIndex) {
		Melody melody1 = new Melody(bpm);
		for(int i=beginIndex; i<endIndex; i++){
				melody1.addNote(new Note(melody.get(i)));
				}

		return melody1;
		
	}

	/**
	 * verhält sich wie public Melody copy(int beginIndex, int endIndex),
	 * erwartet jedoch nur den Parameter beginIndex und gibt eine neue Melodie
	 * bestehend aus den Noten von inklusive beginIndex bis zum Ende der Melodie
	 * zurück.
	 * 
	 * @param beginIndex
	 *            Der erste Index der ersten enthaltenen Note
	 * @return Eine neue Melody aus dem spezifizierten Notenbereich
	 */
	public Melody copy(int beginIndex) {
		return copy(beginIndex, this.melody.size()); // REMOVE THIS
		
	}

	/**
	 * setzt das Tempo der Melodie auf den übergebenen Wert (bpm, Schläge pro
	 * Minute).
	 * 
	 * @param bpm
	 *            Das neue Tempo in Schlägen pro Minuten
	 */
	public void setBPM(int bpm) {
		this.bpm = bpm;
	}

	/**
	 * transponiert alle Noten der Melodie um den gleichen Wert nach oben oder
	 * nach unten. Besteht die Melodie beispielsweise aus den Noten do - mi -
	 * re, so lauten diese nach Aufruf der Methode mit steps=3 fa - la - sol.
	 * Dabei liegt ein zyklischer Abschluss vor: Die sieben Noten lauten do - re
	 * - mi - fa - sol - la - si. Auf die letzte Note si folgt wieder do.
	 * 
	 * @param steps
	 *            Um wieviele Schritte transponiert werden soll
	 */
	public void transpose(int steps) {
		for(int i=0; i<this.melody.size(); i++){
				this.melody.get(i).transpose(steps);
		}
	}

	/**
	 * gibt die Töne der Melodie in der Form Notenname gefolgt von einem
	 * Leerzeichen und der Notenlänge (in Schläge pro Minute) zurück. Zwischen
	 * zwei Noten steht jeweils ein Leerzeichen, alle Noten stehen in der
	 * gleichen Zeile. In einer neuen Zeile wird die Dauer der Melodie in
	 * Sekunden ausgegeben. Ist die gespeicherte Melodie beispielsweise do 1 re
	 * 1 mi 2 fa 4, ist die Gesamtdauer 8 Schläge. Ist das Tempo der Melodie 80
	 * Schläge pro Minute, so beträgt die Länge 6 Sekunden.
	 * 
	 * do 1 re 1 mi 2 fa 4
	 * 6.0 seconds
	 * 
	 * @return Eine lesbare Repräsentation dieser Melody
	 */
	public String toString() {
		String output = new String();
		double beatz = 0.0;
		int sum = 0;
		for(int i=0; i<this.melody.size(); i++){
			output += melody.get(i).toString() + " ";
			sum += melody.get(i).getBeats();
			}
		output = output+"\n";
		beatz = ((60.0 * sum)/(double)this.bpm);
		return output + beatz; 
		
	}

}
